<?php
namespace Planejamento\Logic\RelatorioPlanejamento;

use Geral\Entities\Lista;
use Geral\Entities\Nutriente;
use Geral\Entities\Servico;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Planejamento\Entities\Paramplaneja;
use Planejamento\Entities\Tipoprat;
use Planejamento\Logic\RelatorioCustos\RelatorioCustoLogic;
use Planejamento\Repositories\Cardadia;


/**
 * User: erikasouza
 * Date: 09/07/2013
 * Time: 15:28
 * To change this template use File | Settings | File Templates.
 */
class RelatorioPlanejamentoLogic {

    private $nrlista;
    CONST CDSERVFILI ='cdservfili';
    CONST CDTIPOPRATO ='cdtipoprat';
    CONST CDNUTRIENTE ='cdnutriente';


    /**
     * Gera a lista de servicos por filial ou grupo de servico.
     *
     * @param integer  $nrorg      Numero da Organizacao.
     * @param integer  $cdfilial   Codigo da filial.
     * @param integer  $cdgrupserv Codigo do grupo de servico.
     *
     * @return string
     */
    public function geraListaServico($nrorg, $cdfilial, $cdgrupserv = null, $listaServicos = null) {
        $relatorioCusto = new RelatorioCustoLogic();
        return $relatorioCusto->geraListaServico($nrorg, $cdfilial, $cdgrupserv, $listaServicos);
    }


    public static function mtk_showMonthInicial(){
        $date = new \TkDateTime(tk_get_field_value('DTINICIAL'));
        tk_set_field_value('MESINICIAL', ucfirst($date->get(\TkDateTime::MONTH_NAME)));
    }

    public static function mtk_showMonthFinal(){
        $date = new \TkDateTime(tk_get_field_value('DTFINAL'));
        tk_set_field_value('MESFINAL', ucfirst($date->get(\TkDateTime::MONTH_NAME)));
    }


    public static function geraLista($nrorg, array $servicos, $concatServ = false){
        $nrLista = NovoCodigo::geraCodigo($nrorg, 'LISTA');
        $filial = Ambiente::getCdFilial();

        foreach ($servicos as $servico) {
            if(empty($servico)) continue;
            if ($concatServ) $servico = $filial.$servico;
            $lista = new Lista();
            $lista->insere($nrLista, $servico, $nrorg);
        }

        return $nrLista;
    }


    public static function retornaNomeSgParamPlaneja($filial){
        /** @var $paramPlanejaRepos \Planejamento\Repositories\Paramplaneja */
        $paramPlanejaRepos = Paramplaneja::getRepository();
        $dados = $paramPlanejaRepos->retornaNomeSiglaParamPlaneja($filial);
        return $dados;
    }


    public static function geraListaServicoConcatFilial(){
        $servico     = tk_get_field_value('CDSERVFILI');
        $gruposerv   = tk_get_field_value('CDGRUPSERV');
        $filial      = tk_get_field_value('CDFILIAL');
        $nrorg       = Ambiente::getNrorg();
        $stListaServ = null;

        // Se nao selecionar servico e grupo de servico monta a string com todos servicos da filial.
        if ($servico == "A" && empty($gruposerv) === true) {
            $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ?',
                array($nrorg,$filial,)
            );
            if (empty($servicos) === false) {
                $stListaServ = '_';
                foreach ($servicos as $servico) {
                    $stListaServ .= $filial.$servico->getCdservfili().'_';
                }
            }

        // Se selecionar apenas grupo monta a string com os servicos deste grupo.
        } else if ($servico == "A" && empty($gruposerv) === false) {
            $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND CDGRUPSERV = ?',
                array($nrorg,$filial,$gruposerv)
            );
            if (empty($servicos) === false) {
                $stListaServ = '_';
                foreach ($servicos as $servico) {
                    $stListaServ .= $filial.$servico->getCdservfili().'_';
                }
            }

        // Se selecionar o servico especifico.
        } else if (is_array($servico) === true){
            $stListaServ = '_';
            foreach ($servico as $listaServico)
                foreach ($listaServico as $indice=>$cdServico)
                    if($indice == self::CDSERVFILI)
                        $stListaServ .= $filial.$cdServico.'_';
        } else
            $stListaServ = "";


        return $stListaServ;
    }

    public static function geraListaTpPrato(){
        $tpPrato = tk_get_field_value('CDTIPOPRATO');
        $nrorg = Ambiente::getNrorg();
        $stListaTpPrato = '';
        // Se escolher todos os tipos de prato
        if ($tpPrato == "A") {
            $tpPratos = Tipoprat::getObjectsByWhere(
                'NRORG = ?', array($nrorg)
            );
            if (empty($tpPratos) === false) {
                $stListaTpPrato = '_';

                /** @var $tipo Tipoprat */
                foreach ($tpPratos as $tipo) {
                    $stListaTpPrato .= $tipo->getCdtipoprat().'_';
                }
            }

        // Se não selecionar nenhum tipo de prato.
        } else if ($tpPrato == "N") {
            $stListaTpPrato = 'T';

        // Se selecionar o tipo de prato especifico.
        } else if (is_array($tpPrato) === true){
            $stListaTpPrato = '_';
            foreach ($tpPrato as $listaTpPrato) {
                foreach ($listaTpPrato as $indice=>$cdTpPrato) {
                    if($indice == self::CDTIPOPRATO) {
                        $stListaTpPrato .= $cdTpPrato.'_';
                    }
                }
            }
        }

        return $stListaTpPrato;
    }


    public static function geraListaNutrientes(){
        $nutrientes = tk_get_field_value('CDNUTRIENTE');
        $nrorg = Ambiente::getNrorg();
        $stListaNutriente = '';
        // Se escolher todos os tipos de prato
        if ($nutrientes == "A") {
            $nutriente = Nutriente::getObjectsByWhere(
                'NRORG = ?', array($nrorg)
            );
            if (empty($nutriente) === false) {
                $stListaNutriente = '_';

                /** @var $nutri Nutriente */
                foreach ($nutriente as $nutri) {
                    $stListaNutriente .= $nutri->getCdnutriente().'_';
                }
            }

        // Se selecionar o tipo de prato especifico.
        } else if (is_array($nutrientes) === true){
            $stListaNutriente = '_';
            foreach ($nutrientes as $listaNutri)
                foreach ($listaNutri as $indice=>$cdNutri)
                    if($indice == self::CDNUTRIENTE)
                        $stListaNutriente .= $cdNutri.'_';
        } else
            $stListaNutriente = '';

        return $stListaNutriente;
    }


    public static function geraListaServicoConcatFiliais(){
        $servico     = tk_get_field_value('CDSERVFILI');
        $filial      = tk_get_field_value('CDFILIAL');
        $nrorg       = Ambiente::getNrorg();
        $stListaServ = '_';
        // Se nao selecionar servico e filial monta a string com todos servicos de todas as filial.
        if ($servico == "A" && $filial == "A") {
            /** @var  $filia \Geral\Entities\Filial[]*/
            $filia = \Geral\Entities\Filial::getObjectsByWhere(
                'NRORG = ?',
                array($nrorg)
            );
            foreach($filia as $fili){
                /** @var  $servicos \Geral\Entities\Servico[]*/
                $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                    'NRORG = ? AND CDFILIAL = ?',
                    array($nrorg, $fili->getCdfilial())
                );
                if (empty($servicos) === false) {
                    foreach ($servicos as $servico) {
                        $stListaServ .= $fili->getCdfilial().$servico->getCdservfili().'_';
                    }
                }
            }

            // Se selecionar apenas filial monta a string com os servicos desta filial.
        } else if ($servico == "A" && is_array($filial) === true) {
            foreach($filial as $fil){
                foreach($fil as $indice=>$fili){
                    if($indice == 'cdfilial'){
                        /** @var  $servicos \Geral\Entities\Servico[]*/
                        $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                            'NRORG = ? AND CDFILIAL = ?',
                            array($nrorg, $fili)
                        );
                        if (empty($servicos) === false) {
                            foreach ($servicos as $servico) {
                                $stListaServ .= $fili.$servico->getCdservfili().'_';
                            }
                        }
                    }
                }
            }

            // Se selecionar o servico especifico.
        } else if (is_array($servico) === true){
            //tk_var_dump($servico);
            foreach($servico as $servic){
                $stListaServ .= $servic['cdfilial'].$servic['cdservfili'].'_';
            }
            //tk_var_dump($stListaServ);
        }else
            $stListaServ = "";

        return $stListaServ;
    }


    public static function retoraListaServico($cdServico){
        $listaServico = '';
        if(is_array($cdServico)){
            foreach($cdServico as $servico){
                $listaServico .= $servico['cdservfili'].'_';
            }
        } else{
            $listaServico = $cdServico;
        }

        return $listaServico;
    }


    public static function relatorioCardapioSemana($cardapios,$dtinicial){
        $listaSemana = array();
        $semanas = array();
        $servicos = array();
        $dataSemana = DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtinicial,DateUtil::FORMATO_BRASILEIRO,true),7);

        foreach($cardapios as $cardapio){
            $menorData = DateUtil::retornaMenorData(DateUtil::getDataDeString($cardapio['DTCARDAPIO'],DateUtil::FORMATO_BRASILEIRO_DATAHORA),$dataSemana);
            //Pega o periodo da semana
            if($menorData != $dataSemana){
                array_push($semanas,$cardapio);
                array_push($servicos,$cardapio['CDSERVFILI']);
            } else{
                $dtfinal = DateUtil::truncateString(DateUtil::subtraiIntervalo($dataSemana,1));
                $listaSemana = self::separaSemanaServico($servicos,$semanas,$listaSemana,$dtinicial,$dtfinal);
                $dtinicial = DateUtil::truncateString($dataSemana);

                $semanas = array();
                $servicos = array();
                array_push($semanas,$cardapio);
                array_push($servicos,$cardapio['CDSERVFILI']);
                $dataSemana = DateUtil::adicionaIntervalo($dataSemana,7);
            }
        }
        $dtfinal = DateUtil::truncateString(DateUtil::subtraiIntervalo($dataSemana,1));
        $listaSemana = self::separaSemanaServico($servicos,$semanas,$listaSemana,$dtinicial,$dtfinal);

        return $listaSemana;
    }


    public static function separaSemanaServico($servicos,$semanas,$listaSemana,$dtinicial,$dtfinal){
        $tiposPrato = array();
        $semanaServico = array();

        $servicos = array_unique($servicos);
        sort($servicos);
        foreach($servicos as $servico){
            foreach($semanas as $semana){
                if($servico == $semana['CDSERVFILI']){
                    array_push($semanaServico,$semana);
                    $tiposPrato[$semana['NRSEQPRATCAR']] = $semana['CDTIPOPRAT'];
                }
            }
            $listaSemana = self::montaSemanaRelatorio($tiposPrato,$semanaServico,$listaSemana,$dtinicial,$dtfinal);
            $semanaServico = array();
            $tiposPrato = array();
        }

        return $listaSemana;
    }


    public static function montaSemanaRelatorio($tiposPrato,$semanaServico,$listaSemana,$dtinicial,$dtfinal){
        ksort($tiposPrato);
        foreach($tiposPrato as $tipoPrato){
            $relatorioLinha = array('SERVICO'=>'','TIPOPRATO'=>'','SEG'=>'','TER'=>'','QUA'=>'','QUI'=>'','SEX'=>'','SAB'=>'','DOM'=>'');
            foreach($semanaServico as $tipo){
                if($tipoPrato == $tipo['CDTIPOPRAT']){
                    $relatorioLinha['SERVICO'] = $tipo['CDSERVFILI'].' - '.$tipo['NMSERVICO'].': '.$dtinicial.' - '.$dtfinal;
                    $relatorioLinha['TIPOPRATO'] = $tipo['NMTIPOPRAT'];
                    switch ($tipo['DIASEMANA']){
                        case 1:
                            $relatorioLinha['DOM'] = $tipo['NMPRATO'];
                            break;
                        case 2:
                            $relatorioLinha['SEG'] = $tipo['NMPRATO'];
                            break;
                        case 3:
                            $relatorioLinha['TER'] = $tipo['NMPRATO'];
                            break;
                        case 4:
                            $relatorioLinha['QUA'] = $tipo['NMPRATO'];
                            break;
                        case 5:
                            $relatorioLinha['QUI'] = $tipo['NMPRATO'];
                            break;
                        case 6:
                            $relatorioLinha['SEX'] = $tipo['NMPRATO'];
                            break;
                        case 7:
                            $relatorioLinha['SAB'] = $tipo['NMPRATO'];
                            break;
                    }
                }
            }
            if(!empty($relatorioLinha['TIPOPRATO'])) {
                array_push($listaSemana, $relatorioLinha);
            }
        }

        return $listaSemana;
    }



}
